var SVG_NAMESPACE = 'http://www.w3.org/2000/svg';

var slice = [].slice;

// just a minifier friendly indirection
function createDocumentFragment(document) {
  return document.createDocumentFragment();
}

// given a node, it does what is says
function createText(node, text) {
  return node.ownerDocument.createTextNode(text);
}

var featureFragment = createDocumentFragment(document);

// use native .append(...childNodes) where available
var appendNodes = 'append' in featureFragment ? function appendNodes(node, childNodes) {
  node.append.apply(node, childNodes);
} : function appendNodes(node, childNodes) {
  for (var i = 0, length = childNodes.length; i < length; i++) {
    node.appendChild(childNodes[i]);
  }
};

var HAS_IMPORT_NODE = 'importNode' in document;

// Fix for IE
var REMOVES_EMPTY_TEXTNODES_ON_CLONE = function () {
  featureFragment.appendChild(createText(featureFragment, 'g'));
  featureFragment.appendChild(createText(featureFragment, ''));
  var clone = HAS_IMPORT_NODE ? document.importNode(featureFragment, true) : featureFragment.cloneNode(true);
  return clone.childNodes.length === 1;
}();

var cloneNode = REMOVES_EMPTY_TEXTNODES_ON_CLONE ? function cloneNode(node) {
  var clone = node.cloneNode();
  var childNodes = node.childNodes || [];
  for (var i = 0, l = childNodes.length; i < l; i++) {
    clone.appendChild(cloneNode(childNodes[i]));
  }
  return clone;
} : function (node) {
  return node.cloneNode(true);
};

var importNode = HAS_IMPORT_NODE && !REMOVES_EMPTY_TEXTNODES_ON_CLONE ? function (document, node) {
  return document.importNode(node, true);
} : function (_, node) {
  return cloneNode(node);
};

var NODE_FILTER = NodeFilter.SHOW_ELEMENT | NodeFilter.SHOW_COMMENT | NodeFilter.SHOW_TEXT;

var createTreeWalker = function createTreeWalker(node) {
  return node.ownerDocument.createTreeWalker(node, NODE_FILTER, null, false);
};

// TODO: `createFragment` has some uniqueness, maybe put it somewhere else?

// given a node, inject some html and return
// the resulting template document fragment
function createFragment(document, svg, html) {
  return (svg ? createSVGFragment : createHTMLFragment)(document, html);
}

var SUPPORTS_TEMPLATE = 'content' in document.createElement('template');

// create fragment for HTML
var createHTMLFragment = SUPPORTS_TEMPLATE ? function createHTMLFragment(document, html) {
  var container = document.createElement('template');
  container.innerHTML = html;
  return container.content;
} : function createHTMLFragmentLegacy(document, html) {
  var fragment = createDocumentFragment(document);
  var container = document.createElement('div');

  // el.innerHTML = '<td></td>'; is not possible
  // if the content is a partial internal table content
  // it needs to be wrapped around once injected.
  // HTMLTemplateElement does not suffer this issue.
  var needsTableWrap = /^[^\S]*?<(col(?:group)?|t(?:head|body|foot|r|d|h))/i.test(html);

  if (needsTableWrap) {
    // secure the RegExp.$1 result ASAP to avoid issues
    // in case a non-browser DOM library uses RegExp internally
    // when HTML content is injected (basicHTML / jsdom / others...)
    var selector = RegExp.$1;
    container.innerHTML = '<table>' + html + '</table>';
    appendNodes(fragment, slice.call(container.querySelectorAll(selector)));
  } else {
    container.innerHTML = html;
    appendNodes(fragment, slice.call(container.childNodes));
  }

  return fragment;
};

// create a fragment for SVG
var createSVGFragment = SUPPORTS_TEMPLATE ? function createSVGFragment(document, html) {
  var container = document.createElement('template');
  var svgElement = document.createElementNS(SVG_NAMESPACE, 'svg');
  svgElement.innerHTML = html;
  appendNodes(container.content, slice.call(svgElement.childNodes));
  return container.content;
} : function createSVGFragmentLegacy(document, html) {
  var fragment = createDocumentFragment(document);
  var container = document.createElement('div');
  container.innerHTML = '<svg xmlns="' + SVG_NAMESPACE + '">' + html + '</svg>';
  appendNodes(fragment, slice.call(container.firstChild.childNodes));
  return fragment;
};

var EXPANDO = '_hyper_';
var UID = EXPANDO + (Math.random() * new Date() | 0) + ';';
var UIDC = '<!--' + UID + '-->';

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) {
  return typeof obj;
} : function (obj) {
  return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj;
};











var classCallCheck = function (instance, Constructor) {
  if (!(instance instanceof Constructor)) {
    throw new TypeError("Cannot call a class as a function");
  }
};

var createClass = function () {
  function defineProperties(target, props) {
    for (var i = 0; i < props.length; i++) {
      var descriptor = props[i];
      descriptor.enumerable = descriptor.enumerable || false;
      descriptor.configurable = true;
      if ("value" in descriptor) descriptor.writable = true;
      Object.defineProperty(target, descriptor.key, descriptor);
    }
  }

  return function (Constructor, protoProps, staticProps) {
    if (protoProps) defineProperties(Constructor.prototype, protoProps);
    if (staticProps) defineProperties(Constructor, staticProps);
    return Constructor;
  };
}();









var inherits = function (subClass, superClass) {
  if (typeof superClass !== "function" && superClass !== null) {
    throw new TypeError("Super expression must either be null or a function, not " + typeof superClass);
  }

  subClass.prototype = Object.create(superClass && superClass.prototype, {
    constructor: {
      value: subClass,
      enumerable: false,
      writable: true,
      configurable: true
    }
  });
  if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass;
};











var possibleConstructorReturn = function (self, call) {
  if (!self) {
    throw new ReferenceError("this hasn't been initialised - super() hasn't been called");
  }

  return call && (typeof call === "object" || typeof call === "function") ? call : self;
};





var slicedToArray = function () {
  function sliceIterator(arr, i) {
    var _arr = [];
    var _n = true;
    var _d = false;
    var _e = undefined;

    try {
      for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) {
        _arr.push(_s.value);

        if (i && _arr.length === i) break;
      }
    } catch (err) {
      _d = true;
      _e = err;
    } finally {
      try {
        if (!_n && _i["return"]) _i["return"]();
      } finally {
        if (_d) throw _e;
      }
    }

    return _arr;
  }

  return function (arr, i) {
    if (Array.isArray(arr)) {
      return arr;
    } else if (Symbol.iterator in Object(arr)) {
      return sliceIterator(arr, i);
    } else {
      throw new TypeError("Invalid attempt to destructure non-iterable instance");
    }
  };
}();



var taggedTemplateLiteral = function (strings, raw) {
  return Object.freeze(Object.defineProperties(strings, {
    raw: {
      value: Object.freeze(raw)
    }
  }));
};









var toConsumableArray = function (arr) {
  if (Array.isArray(arr)) {
    for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) arr2[i] = arr[i];

    return arr2;
  } else {
    return Array.from(arr);
  }
};

var ELEMENT_NODE = 1;
var ATTRIBUTE_NODE = 2;
var TEXT_NODE = 3;
var COMMENT_NODE = 8;

// eslint-disable-next-line no-control-regex
var EXTRACT_ATTRIBUTE_NAME = /\s([^\0-\x1F\x7F-\x9F \x09\x0a\x0c\x0d"'>=/]+)[ \x09\x0a\x0c\x0d]*=[ \x09\x0a\x0c\x0d]*['"]?$/;
var WHITESPACE_NODE_TEST = /^\n\s*$/;

/*
  Given the unique static strings of a template-tag invocation,
  create a blueprint fragment and notes about its dynamic parts
  which we can use over and over for new instances of this (template)
  fragment.
 */
function createTemplateBlueprint$1(nodeCallback, attributeCallback, strings, document, isSvg) {
  var _processStrings = processStrings(strings, nodeCallback, attributeCallback),
      html = _processStrings.html,
      notes = _processStrings.notes;

  var fragment = createFragment(document, isSvg, html);
  return processFragment(notes, fragment);
}

/*
  The functions `findTagClose` and `processStrings` originally come from the
  lit-html PR https://github.com/PolymerLabs/lit-html/pull/153
  authored by https://github.com/jridgewell

  Slightly modified to actually work.
 */

/*
  Finds the closing index of the last closed HTML tag.
  This has 3 possible return values:
    - `-1`, meaning there is no tag in str.
    - `string.length`, meaning the last opened tag is unclosed.
    - Some positive number < str.length, meaning the index of the closing '>'.
 */
function findTagClose(str) {
  var close = str.lastIndexOf('>');
  var open = str.indexOf('<', close + 1);
  return open > -1 ? str.length : close;
}

// In general, walk the static strings the user gave us, and decide which
// 'hole'/interpolation will be on an attribute or is in a node position.
// Prepare the HTML we will actually use as a template. Return the HTML
// and notes about each 'hole' we found.
var processStrings = function processStrings(strings, nodeCallback, attributeCallback) {
  var nodeMarker = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : UIDC;
  var attributeMarker = arguments.length > 4 && arguments[4] !== undefined ? arguments[4] : UID;

  var last = strings.length - 1;
  var html = '';
  var notes = [];

  var isTextBinding = true;
  for (var i = 0; i < last; i++) {
    var s = strings[i];
    html += s;
    // We're in a text position if the previous string closed its tags.
    // If it doesn't have any tags, then we use the previous text position
    // state.
    var closing = findTagClose(s);
    isTextBinding = closing > -1 ? closing < s.length : isTextBinding;
    if (isTextBinding) {
      html += nodeMarker;
      notes.push({ type: ELEMENT_NODE, updater: nodeCallback });
    } else {
      html += attributeMarker;
      // In general attributes are stored in `node.attributes` as a map, which
      // means they're unordered. But user's `strings` is ordered and we MUST
      // retain this order. We therefore extract the wanted attributeName from
      // the current string (`s`).
      var name = extractAttributeName(s);
      notes.push({ type: ATTRIBUTE_NODE, updater: attributeCallback(name) });
    }
  }
  html += strings[last];
  return { html: html, notes: notes };
};

var extractAttributeName = function extractAttributeName(string) {
  var match = EXTRACT_ATTRIBUTE_NAME.exec(string);
  if (!match) {
    throw new Error('Could not get the attribute name within the following String \'' + string + '\'');
  }

  return match[1];
};

var processFragment = function processFragment(firstNotes, fragment) {
  var nodeMarker = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : UIDC;
  var marker = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : UID;

  var notes = [];
  var nextNode = domWalker(fragment, nodeMarker, marker);
  // We keep an indexOffset here in case we create text nodes the walker did
  // not count.
  var indexOffset = 0;

  // We HAVE to remove each attribute we will find at the end of this fn,
  // otherwise the user would see our markers.
  var foundAttributes = [];

  for (var i = 0, l = firstNotes.length; i < l; i++) {
    var earlyNote = firstNotes[i];
    var _earlyNote = earlyNote,
        typeHint = _earlyNote.type,
        updater = _earlyNote.updater;

    var _nextNode = nextNode(typeHint),
        _nextNode2 = slicedToArray(_nextNode, 2),
        node = _nextNode2[0],
        index = _nextNode2[1];

    switch (node.nodeType) {
      case ATTRIBUTE_NODE:
        notes.push({ index: index + indexOffset, updater: updater });
        foundAttributes.push(node);
        break;
      case COMMENT_NODE:
        notes.push({ index: index + indexOffset, updater: updater });
        break;
      case TEXT_NODE:
        {
          // Whenever we encounter a TEXT_NODE, the browser actually failed at
          // inserting a COMMENT_NODE, because that's what we asked for. This
          // is the case e.g. for <style> tags, which can only have TEXT_NODEs
          // as children.

          // Now let the input be html`<style>a${}b${}c</style>`. The walker
          // sees exactly ONE text node (with {{~}} denoting our marker):
          //   `a{{~}}b${{~}}c`
          // We have to fulfill TWO notes and since the node updater always
          // controls and fills the space BEFORE him, we somehow must end up
          // with three text nodes 'a', 'b' and 'c'; and at the same time push
          // two notes referring to 'b' and 'c'.

          var document = node.ownerDocument;
          var parentNode = node.parentNode;

          // We split the string into its static parts.
          var value = node.nodeValue;
          var parts = value.split(nodeMarker);

          // For all parts except the last one, we create a new text node, and
          // a note
          var last = parts.length - 1;
          for (var j = 0; j < last; j++) {
            var textNode = document.createTextNode(parts[j]);
            // `insertBefore` to not move the position of the domWalker
            parentNode.insertBefore(textNode, node);

            // Create a note pointing to the *next* node
            indexOffset++;
            notes.push({
              index: index + indexOffset,
              updater: earlyNote.updater
            });
            // ATT: We MUST forward the outer for-loop
            earlyNote = firstNotes[++i];
          }

          // Reuse the existing node for the last part.
          node.nodeValue = parts[last];
        }
    }
  }

  // Completely walk the DOM, and remove some whitespace nodes.
  nextNode();

  for (var _i = 0, _l = foundAttributes.length; _i < _l; _i++) {
    var attribute = foundAttributes[_i];
    var _node = attribute.ownerElement;
    _node.removeAttributeNode(attribute);
  }

  return { fragment: fragment, notes: notes };
};

// This is like a normal `TreeWalker` but yields only nodes with markers.
// The returned `next` function accepts a `typeHint` for optimization purposes.
// Say, we expect the next node to be a 'node', we can just skip processing
// attributes.
var domWalker = function domWalker(node, nodeMarker, marker) {
  var walker = createTreeWalker(node);
  var index = -1;

  var attributes = null;
  var attrIndex = 0;

  var whitespaceOnlyNodes = [];

  return function next(typeHint) {
    while (true) {
      if (attributes && typeHint === ATTRIBUTE_NODE) {
        for (var l = attributes.length; attrIndex < l; attrIndex++) {
          var attr = attributes[attrIndex];
          if (attr.value === marker) {
            // ATT: We MUST increment the for-loop var manually before
            // returning
            attrIndex++;
            return [attr, index];
          }
        }

        attributes = null;
      }

      WALKER: {
        while (walker.nextNode()) {
          index++;
          var _node2 = walker.currentNode;

          switch (_node2.nodeType) {
            case ELEMENT_NODE:
              if (typeHint === ATTRIBUTE_NODE && _node2.hasAttributes()) {
                attributes = _node2.attributes;
                attrIndex = 0;
                break WALKER;
              }
              continue;
            case COMMENT_NODE:
              if (_node2.nodeValue === marker) {
                return [_node2, index];
              }
              continue;
            case TEXT_NODE:
              {
                var nodeValue = _node2.nodeValue;
                if (WHITESPACE_NODE_TEST.test(nodeValue)) {
                  index--;
                  whitespaceOnlyNodes.push(_node2);
                } else if (nodeValue.indexOf(nodeMarker) !== -1) {
                  return [_node2, index];
                }
                continue;
              }
          }
        }

        for (var i = 0, _l2 = whitespaceOnlyNodes.length; i < _l2; i++) {
          var _node3 = whitespaceOnlyNodes[i];
          _node3.parentNode.removeChild(_node3);
        }
        return;
      }
    }
  };
};

var createTemplateInstance = function createTemplateInstance(createTemplateBlueprint, document, isSvg, strings) {
  var _createTemplateBluepr = createTemplateBlueprint(strings, document, isSvg),
      fragment = _createTemplateBluepr.fragment,
      notes = _createTemplateBluepr.notes;

  return instantiateBlueprint(document, fragment, notes);
};

var instantiateBlueprint = function instantiateBlueprint(document, blueprintFragment, notes) {
  var fragment = importNode(document, blueprintFragment);
  var updaters = readyUpdaters(fragment, notes);
  return { fragment: fragment, updaters: updaters };
};

// Find the correct live node using our notes, and pass it to the
// updater fns. Return just the updaters.
function readyUpdaters(fragment, notes) {
  var walker = createTreeWalker(fragment);
  var index = -1;
  var updates = [];
  for (var i = 0, l = notes.length; i < l; i++) {
    var _notes$i = notes[i],
        updater = _notes$i.updater,
        nodeIndex = _notes$i.index;


    while (index < nodeIndex) {
      index++;
      walker.nextNode();
    }

    updates[i] = updater(walker.currentNode);
  }
  return updates;
}

/* Pseudo Polyfill. Just enough to get it working. */

// Map with partial double Array fallback
var Map = self.Map || function () {
  var k = [],
      v = [];
  return {
    get: function get(obj) {
      return v[k.indexOf(obj)];
    },
    // being used with unique template literals
    // there is never a case when a value is overwritten
    // no need to check upfront for the indexOf
    set: function set(obj, value) {
      v[k.push(obj) - 1] = value;
    }
  };
};

var memoizeOnFirstArg = function memoizeOnFirstArg(fn) {
  var cache = new Map();
  return function (arg) {
    for (var _len = arguments.length, args = Array(_len > 1 ? _len - 1 : 0), _key = 1; _key < _len; _key++) {
      args[_key - 1] = arguments[_key];
    }

    var rv = cache.get(arg);
    if (rv === undefined) {
      rv = fn.apply(undefined, [arg].concat(args));
      cache.set(arg, rv);
    }
    return rv;
  };
};

var lruCacheOne = function lruCacheOne(fn) {
  var lastIn = void 0,
      lastOut = void 0;
  return function (arg) {
    for (var _len2 = arguments.length, args = Array(_len2 > 1 ? _len2 - 1 : 0), _key2 = 1; _key2 < _len2; _key2++) {
      args[_key2 - 1] = arguments[_key2];
    }

    if (lastIn !== arg) {
      lastIn = arg;
      lastOut = fn.apply(undefined, [arg].concat(args));
    }
    return lastOut;
  };
};

var flatten = function flatten(array) {
  var _ref;

  return (_ref = []).concat.apply(_ref, toConsumableArray(array));
};

// used to convert childNodes to Array
var slice$1 = [].slice;

// older WebKit need this


// BB7 and webOS need this
var isArray = Array.isArray || function () {
  var toString = {}.toString;
  // I once had an engine returning [array Array]
  // and I've got scared since!
  var s = toString.call([]);
  return function (a) {
    return toString.call(a) === s;
  };
}();

var isObject = function isObject(x) {
  return x != null && (typeof x === 'undefined' ? 'undefined' : _typeof(x)) === 'object';
};

// Keep it stupid simple here
var isObservable = function isObservable(value) {
  return isObject(value) && 'subscribe' in value;
};
var isSubject = function isSubject(value) {
  return isObject(value) && 'next' in value;
};

var _templateObject$1 = taggedTemplateLiteral([''], ['']);

function render(createTemplateInstance, finalSideEffect, strings) {
  strings = _unique(strings);

  var _createTemplateInstan = createTemplateInstance(strings),
      fragment = _createTemplateInstan.fragment,
      updaters = _createTemplateInstan.updaters;

  for (var _len = arguments.length, values = Array(_len > 3 ? _len - 3 : 0), _key = 3; _key < _len; _key++) {
    values[_key - 3] = arguments[_key];
  }

  update(updaters, values);
  return finalSideEffect(fragment);
}

// invokes each update function passing interpolated value
function update(updateFns, values) {
  for (var i = 0, length = updateFns.length; i < length; i++) {
    updateFns[i](values[i]);
  }
}

var replaceNodeContent = function replaceNodeContent(node, fragment) {
  node.textContent = '';
  node.appendChild(fragment);
  return node;
};

// return a single node or an Array or nodes
var extractContent = function extractContent(node) {
  var childNodes = node.childNodes;
  return childNodes.length === 1 ? childNodes[0] : slice$1.call(childNodes);
};

// Both Firefox < 55 and TypeScript have issues with template literals.
// This lazy defined callback should spot issues right away
// and in the best case scenario become a no-op
var _unique = function unique(strings) {
  // ATT: If we inline `fn`, uglify changes the behavior of our code!
  var fn = function fn(strings) {
    return strings;
  };
  var testFn = function testFn() {
    return fn(_templateObject$1);
  };
  if (testFn() === testFn()) {
    _unique = function unique(strings) {
      return strings;
    };
  } else {
    var templateObjects = Object.create(null);
    _unique = function unique(strings) {
      var key = strings.join(UIDC);
      return templateObjects[key] || (templateObjects[key] = strings);
    };
  }
  return _unique(strings);
};

/*! (c) 2017 Andrea Giammarchi (ISC) */

/**
 * This code is a revisited port of the snabbdom vDOM diffing logic,
 * the same that fuels as fork Vue.js or other libraries.
 * @credits https://github.com/snabbdom/snabbdom
 */

var identity = function identity(O) {
  return O;
};

var domdiff = function domdiff(parentNode, // where changes happen
currentNodes, // Array of current items/nodes
futureNodes, // Array of future items/nodes
getNode, // optional way to retrieve a node from an item
beforeNode // optional item/node to use as insertBefore delimiter
) {
  var get = getNode || identity;
  var before = beforeNode == null ? null : get(beforeNode, 0);
  var currentStart = 0,
      futureStart = 0;
  var currentEnd = currentNodes.length - 1;
  var currentStartNode = currentNodes[0];
  var currentEndNode = currentNodes[currentEnd];
  var futureEnd = futureNodes.length - 1;
  var futureStartNode = futureNodes[0];
  var futureEndNode = futureNodes[futureEnd];
  while (currentStart <= currentEnd && futureStart <= futureEnd) {
    if (currentStartNode == null) {
      currentStartNode = currentNodes[++currentStart];
    } else if (currentEndNode == null) {
      currentEndNode = currentNodes[--currentEnd];
    } else if (futureStartNode == null) {
      futureStartNode = futureNodes[++futureStart];
    } else if (futureEndNode == null) {
      futureEndNode = futureNodes[--futureEnd];
    } else if (currentStartNode == futureStartNode) {
      currentStartNode = currentNodes[++currentStart];
      futureStartNode = futureNodes[++futureStart];
    } else if (currentEndNode == futureEndNode) {
      currentEndNode = currentNodes[--currentEnd];
      futureEndNode = futureNodes[--futureEnd];
    } else if (currentStartNode == futureEndNode) {
      parentNode.insertBefore(get(currentStartNode, 1), get(currentEndNode, -0).nextSibling);
      currentStartNode = currentNodes[++currentStart];
      futureEndNode = futureNodes[--futureEnd];
    } else if (currentEndNode == futureStartNode) {
      parentNode.insertBefore(get(currentEndNode, 1), get(currentStartNode, 0));
      currentEndNode = currentNodes[--currentEnd];
      futureStartNode = futureNodes[++futureStart];
    } else {
      var index = currentNodes.indexOf(futureStartNode);
      if (index < 0) {
        parentNode.insertBefore(get(futureStartNode, 1), get(currentStartNode, 0));
        futureStartNode = futureNodes[++futureStart];
      } else {
        var el = currentNodes[index];
        currentNodes[index] = null;
        parentNode.insertBefore(get(el, 1), get(currentStartNode, 0));
        futureStartNode = futureNodes[++futureStart];
      }
    }
  }
  if (currentStart > currentEnd) {
    var pin = futureNodes[futureEnd + 1];
    var place = pin != null ? get(pin, 0) : before;
    while (futureStart <= futureEnd) {
      var ch = futureNodes[futureStart++];
      // ignore until I am sure the else could never happen.
      // it might be a vDOM thing 'cause it never happens here.
      /* istanbul ignore else */
      if (ch != null) parentNode.insertBefore(get(ch, 1), place);
    }
  }
  // ignore until I am sure the else could never happen.
  // it might be a vDOM thing 'cause it never happens here.
  /* istanbul ignore else */
  else if (futureStart > futureEnd) {
      while (currentStart <= currentEnd) {
        var _ch = currentNodes[currentStart++];
        if (_ch != null) parentNode.removeChild(get(_ch, -1));
      }
    }
  return futureNodes;
};

function TagInvocation(strings, values, isSvg, key) {
  this.strings = strings;
  this.values = values;
  this.isSvg = isSvg;
  this.key = key;
}

var TEXT_NODE$1 = 3;
var DOCUMENT_FRAGMENT_NODE = 11;

// transformers registry; shared and global, yeah!
var transformers = {};
var transformersKeys = [];

var NodeHolder = function () {
  function NodeHolder(marker) {
    var managedNodes = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : [];
    classCallCheck(this, NodeHolder);

    this._marker = marker;
    this._managedNodes = managedNodes;
  }

  createClass(NodeHolder, [{
    key: '_optimist',
    value: function _optimist(nextNodes) {
      this._managedNodes = domdiff(this._marker.parentNode, this._managedNodes, nextNodes, null, this._marker);
    }
  }, {
    key: 'setText',
    value: function setText(value) {
      var managedNodes = this._managedNodes;
      var length = managedNodes.length;
      if (length === 1 && managedNodes[0].nodeType === TEXT_NODE$1) {
        // Reuse existing TEXT_NODE
        managedNodes[0].nodeValue = value;
      } else {
        if (length) {
          // Clear everything first, to optimize for the following append
          this.clear();
        }
        this._optimist([createText(this._marker, value)]);
      }
    }
  }, {
    key: 'setHTML',
    value: function setHTML(value) {
      var fragment = createFragment(this._marker.ownerDocument, false, value);
      this._optimist(slice$1.call(fragment.childNodes));
    }
  }, {
    key: 'setNodes',
    value: function setNodes(value) {
      this._optimist(value.nodeType === DOCUMENT_FRAGMENT_NODE ? slice$1.call(value.childNodes) : isArray(value) ? value : [value]);
    }
  }, {
    key: 'appendNodes',
    value: function appendNodes$$1(value) {
      var nodes = this._managedNodes.concat(value);
      this._optimist(nodes);
    }
  }, {
    key: 'appendText',
    value: function appendText(value) {
      this.appendNodes([createText(this._marker, value)]);
    }
  }, {
    key: 'appendHTML',
    value: function appendHTML(value) {
      var fragment = createFragment(this._marker.ownerDocument, false, value);
      this.appendNodes(slice$1.call(fragment.childNodes));
    }
  }, {
    key: 'clear',
    value: function clear() {
      this._optimist([]);
    }
  }]);
  return NodeHolder;
}();

var makeNodeSetter = function makeNodeSetter(nodeMarker) {
  var holder = new NodeHolder(nodeMarker);

  var oldValue = void 0;
  var wires = Object.create(null);

  return function setValue(value) {
    if (value === oldValue) {
      return;
    }
    oldValue = value;

    if (value == null) {
      value = [];
    }

    switch (typeof value === 'undefined' ? 'undefined' : _typeof(value)) {
      case 'string':
      case 'number':
      case 'boolean':
        holder.setText(value);
        return;
    }

    if (isNode(value)) {
      holder.setNodes(value);
      return;
    }

    if (value instanceof TagInvocation) {
      var tagInvocation = value;
      var key = tagInvocation.key || tagInvocation.type;
      var wire$$1 = wires[key] || (wires[key] = materializer());
      holder.setNodes(wire$$1(tagInvocation));
      return;
    }

    if (isArray(value) || 'length' in value && (value = slice$1.call(value))) {
      var length = value.length;

      if (length === 0) {
        wires = Object.create(null);
        holder.clear();
        return;
      }

      switch (_typeof(value[0])) {
        case 'string':
        case 'number':
        case 'boolean':
          holder.setText(value.join(''));
          return;
        case 'object':
          if (isArray(value[0])) {
            setValue(flatten(value));
            return;
          }
          if (isPromise(value[0])) {
            Promise.all(value).then(setValue);
            return;
          }
          if (value[0] instanceof TagInvocation) {
            var newWires = Object.create(null);
            for (var i = 0; i < length; i++) {
              var _tagInvocation = value[i];
              var _key = _tagInvocation.key || i;
              var _wire = newWires[_key] = wires[_key] || materializer();
              value[i] = _wire(_tagInvocation);
            }
            wires = newWires;
            value = flatten(value);
          }
      }

      holder.setNodes(value);
      return;
    }

    if (isPromise(value)) {
      value.then(setValue);
    } else if ('html' in value) {
      holder.setHTML(value.html);
    } else {
      invokeTransformer(value, setValue, holder);
    }
  };
};

// last attempt to transform content
function invokeTransformer(value, setValue, nodeHolder) {
  for (var key, i = 0, length = transformersKeys.length; i < length; i++) {
    key = transformersKeys[i];
    if (value.hasOwnProperty(key)) {
      transformers[key](value[key], setValue, nodeHolder, value);
    }
  }
}

// quick and dirty Node check
function isNode(value) {
  return 'ELEMENT_NODE' in value;
}

// quick and dirty Promise check
function isPromise(value) {
  return 'then' in value;
}

var makeAttributeSetter = function makeAttributeSetter(node, attributeName) {
  var oldValue = void 0;

  return function (value) {
    if (value === true) {
      value = '';
    }

    if (oldValue === value) {
      return;
    }

    oldValue = value;

    if (value == null || value === false) {
      node.removeAttribute(attributeName);
    } else {
      node.setAttribute(attributeName, value);
    }
  };
};

var makePropertySetter = function makePropertySetter(node, propertyName) {
  if (propertyName in node) {
    return makePropertySetterWithDirtyCheck(node, propertyName);
  }
  return makeSimplePropertySetter(node, propertyName);
};

var makeSimplePropertySetter = function makeSimplePropertySetter(node, propertyName) {
  return function (value) {
    node[propertyName] = value;
  };
};

var makePropertySetterWithDirtyCheck = function makePropertySetterWithDirtyCheck(node, propertyName) {
  var oldValue = null;

  return function (value) {
    var valueIsPrimitive = (typeof value === 'undefined' ? 'undefined' : _typeof(value)) != 'object' || value === null;
    if (value == null) {
      value = '';
    }

    if (valueIsPrimitive && oldValue === value && node[propertyName] === value) {
      return;
    }

    oldValue = value;

    node[propertyName] = value;
  };
};

var makeEventHandler = function makeEventHandler(node, eventName) {
  var currentValue = void 0;
  var handler = function handler(ev) {
    return currentValue(ev);
  };

  return function (nextValue) {
    if (currentValue === nextValue) {
      return;
    }

    if (!nextValue) {
      node.removeEventListener(eventName, handler, false);
    } else if (!currentValue) {
      node.addEventListener(eventName, handler, false);
    }

    currentValue = nextValue;
  };
};

var rxAware = function rxAware(fn) {
  return function () {
    var oldValue = void 0,
        subscription = void 0;
    var sink = fn.apply(undefined, arguments);

    return function (value) {
      if (isObservable(value)) {
        if (oldValue === value) {
          return;
        }
        oldValue = value;

        if (subscription) {
          subscription.unsubscribe();
          subscription = null;
        }
        if (value) {
          subscription = value.subscribe(sink);
        }
      } else {
        oldValue = value;

        if (subscription) {
          subscription.unsubscribe();
          subscription = null;
        }

        sink(value);
      }
    };
  };
};

var makeRxAttributeSetter = rxAware(makeAttributeSetter);
var makeRxPropertySetter = rxAware(makePropertySetter);

var makeRxEventHandler = function makeRxEventHandler(node, eventName) {
  var sink = makeEventHandler(node, eventName);

  return function setter(value) {
    if (isSubject(value)) {
      sink(function (ev) {
        return value.next(ev);
      });
    } else {
      sink(value);
    }
  };
};

var attributeCallback = function attributeCallback(attributeName) {
  var _ref = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {},
      _ref$makeAttributeSet = _ref.makeAttributeSetterF,
      makeAttributeSetterF = _ref$makeAttributeSet === undefined ? makeAttributeSetter : _ref$makeAttributeSet,
      _ref$makePropertySett = _ref.makePropertySetterF,
      makePropertySetterF = _ref$makePropertySett === undefined ? makePropertySetter : _ref$makePropertySett,
      _ref$makeEventHandler = _ref.makeEventHandlerF,
      makeEventHandlerF = _ref$makeEventHandler === undefined ? makeEventHandler : _ref$makeEventHandler;

  if (attributeName.startsWith('on')) {
    // Allow `onclick` as well as `on-click`
    var eventName = attributeName[2] === '-' ? attributeName.slice(3) : attributeName.slice(2);
    eventName = eventName.toLowerCase();
    return function (node) {
      return makeEventHandlerF(node, eventName);
    };
  }

  if (attributeName.endsWith('$')) {
    var newName = attributeName.slice(0, -1);
    return function (node) {
      return makeAttributeSetterF(node, newName);
    };
  }

  return function (node) {
    return makePropertySetterF(node, attributeName);
  };
};

var rwAwareNodeCallback = rxAware(makeNodeSetter);

var rxAwareAttributeCallback = function rxAwareAttributeCallback(name) {
  return attributeCallback(name, {
    makeAttributeSetterF: makeRxAttributeSetter,
    makePropertySetterF: makeRxPropertySetter,
    makeEventHandlerF: makeRxEventHandler
  });
};

/*! (c) 2017 Andrea Giammarchi @WebReflection, (ISC) */
/*! (c) 2017 herr.kaste, (ISC) */

var OWNER_SVG_ELEMENT = 'ownerSVGElement';

var createTemplateBlueprint$$1 = memoizeOnFirstArg(createTemplateBlueprint$1.bind(null, rwAwareNodeCallback, rxAwareAttributeCallback));

var upgrade = createTemplateInstance.bind(null, createTemplateBlueprint$$1);

// The main render factories

// render TL inside a DOM node used as context
var bind = function bind(node) {
  var finalSideEffect = lruCacheOne(replaceNodeContent.bind(null, node));

  var document = node.ownerDocument;
  var isSvg = OWNER_SVG_ELEMENT in node;
  var upgrader = memoizeOnFirstArg(upgrade.bind(null, document, isSvg));

  return render.bind(null, upgrader, finalSideEffect);
};

var html = function html(strings) {
  for (var _len = arguments.length, values = Array(_len > 1 ? _len - 1 : 0), _key = 1; _key < _len; _key++) {
    values[_key - 1] = arguments[_key];
  }

  if (isArray(strings)) {
    return new TagInvocation(strings, values, false);
  }
  var key = strings;
  return function (strings) {
    for (var _len2 = arguments.length, values = Array(_len2 > 1 ? _len2 - 1 : 0), _key2 = 1; _key2 < _len2; _key2++) {
      values[_key2 - 1] = arguments[_key2];
    }

    return new TagInvocation(strings, values, false, key);
  };
};



var materialize = function materialize(tagInvocation) {
  return materializer()(tagInvocation);
};

var materializer = function materializer() {
  var finalSideEffect = lruCacheOne(extractContent);
  var wire = lruCacheOne(function (isSvg) {
    return lruCacheOne(upgrade.bind(null, document, isSvg));
  });
  return function (tagInvocation) {
    var strings = tagInvocation.strings,
        values = tagInvocation.values,
        isSvg = tagInvocation.isSvg;

    var upgrader = wire(isSvg);
    return render.apply(undefined, [upgrader, finalSideEffect, strings].concat(toConsumableArray(values)));
  };
};

var keyed = function keyed(key, tagInvocation) {
  var strings = tagInvocation.strings,
      values = tagInvocation.values,
      isSvg = tagInvocation.isSvg;

  return new TagInvocation(strings, values, isSvg, key);
};

function wire() {
  var type = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : 'html';

  switch (type) {
    case 'html':
      return wireHtml();
    case 'svg':
      return wireSvg();
  }
}

var wireHtml = function wireHtml() {
  var finalSideEffect = lruCacheOne(extractContent);
  var upgrader = memoizeOnFirstArg(upgrade.bind(null, document, false));
  return render.bind(null, upgrader, finalSideEffect);
};

var wireSvg = function wireSvg() {
  var finalSideEffect = lruCacheOne(extractContent);
  var upgrader = memoizeOnFirstArg(upgrade.bind(null, document, true));
  return render.bind(null, upgrader, finalSideEffect);
};

// - - - - - - - - - - - - - - - - - - - - - - -

// hyper.define('transformer', callback) 🌀

/* global chai */

var _chai$1 = chai;
var assert$1 = _chai$1.assert;


var matchInnerHTML = function matchInnerHTML(string, html) {
  var container = html.cloneNode(true);
  removeComments(container);
  assert$1.equal(string, container.innerHTML);
};

var removeComments = function removeComments(node) {
  var walker = document.createTreeWalker(node, NodeFilter.SHOW_COMMENT, null, false);
  var commentNodes = [];

  while (walker.nextNode()) {
    commentNodes.push(walker.currentNode);
  }

  var _iteratorNormalCompletion = true;
  var _didIteratorError = false;
  var _iteratorError = undefined;

  try {
    for (var _iterator = commentNodes[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
      var n = _step.value;

      n.parentNode.removeChild(n);
    }
  } catch (err) {
    _didIteratorError = true;
    _iteratorError = err;
  } finally {
    try {
      if (!_iteratorNormalCompletion && _iterator.return) {
        _iterator.return();
      }
    } finally {
      if (_didIteratorError) {
        throw _iteratorError;
      }
    }
  }
};

var _templateObject = taggedTemplateLiteral(['<p>Hello World!</p>'], ['<p>Hello World!</p>']);
var _templateObject2 = taggedTemplateLiteral(['<p>', '</p>'], ['<p>', '</p>']);
var _templateObject3 = taggedTemplateLiteral(['', ''], ['', '']);
var _templateObject4 = taggedTemplateLiteral(['', '<p>', '</p>', '<hr>', ''], ['', '<p>', '</p>', '<hr>', '']);
var _templateObject5 = taggedTemplateLiteral(['<p> ', ' ', ' </p>'], ['<p> ', ' ', ' </p>']);
var _templateObject6 = taggedTemplateLiteral(['\n        <p>Hello</p>\n        <p>Friend</p>\n      '], ['\n        <p>Hello</p>\n        <p>Friend</p>\n      ']);
var _templateObject7 = taggedTemplateLiteral(['\n        <p>Hello</p>\n      '], ['\n        <p>Hello</p>\n      ']);
var _templateObject8 = taggedTemplateLiteral(['\n        <p>Hello</p> <p>Hello</p>\n      '], ['\n        <p>Hello</p> <p>Hello</p>\n      ']);
var _templateObject9 = taggedTemplateLiteral(['<style>', '</style>'], ['<style>', '</style>']);
var _templateObject10 = taggedTemplateLiteral(['<style>Hello', 'Friend</style>'], ['<style>Hello', 'Friend</style>']);
var _templateObject11 = taggedTemplateLiteral(['<style>', 'X', 'X', 'X', '</style>'], ['<style>', 'X', 'X', 'X', '</style>']);
var _templateObject12 = taggedTemplateLiteral(['<textarea>', '</textarea>'], ['<textarea>', '</textarea>']);
var _templateObject13 = taggedTemplateLiteral(['<p some-attr$=', '></p>'], ['<p some-attr$=', '></p>']);
var _templateObject14 = taggedTemplateLiteral(['<p some-attr$="', '"></p>'], ['<p some-attr$="', '"></p>']);
var _templateObject15 = taggedTemplateLiteral(['<p hidden$=', '></p>'], ['<p hidden$=', '></p>']);
var _templateObject16 = taggedTemplateLiteral(['<p align=', '></p>'], ['<p align=', '></p>']);
var _templateObject17 = taggedTemplateLiteral(['<p onclick=', '></p>'], ['<p onclick=', '></p>']);
var _templateObject18 = taggedTemplateLiteral(['<h1>', '</h1>'], ['<h1>', '</h1>']);
var _templateObject19 = taggedTemplateLiteral(['<x-foo></x-foo>'], ['<x-foo></x-foo>']);
var _templateObject20 = taggedTemplateLiteral(['<x-bar foo$=', '></x-bar>'], ['<x-bar foo$=', '></x-bar>']);

/* global chai */

var _chai = chai;
var assert = _chai.assert;


describe('bind/render', function () {
  var div = void 0;

  beforeEach(function () {
    div = document.createElement('div');
    document.body.appendChild(div);
  });

  afterEach(function () {
    document.body.removeChild(div);
  });

  it('renders static html', function () {
    var render = bind(div);
    render(_templateObject);

    assert.equal('<p>Hello World!</p>', div.innerHTML);
  });

  it('renders a number', function () {
    var render = bind(div);
    render(_templateObject2, 123);
    matchInnerHTML('<p>' + 123 + '</p>', div);
  });

  it('renders a string', function () {
    var render = bind(div);
    render(_templateObject2, 'bar');
    matchInnerHTML('<p>' + 'bar' + '</p>', div);
  });

  it('renders `undefined` as empty string', function () {
    var render = bind(div);
    render(_templateObject2, undefined);
    matchInnerHTML('<p></p>', div);
  });

  it('renders `null` as empty string', function () {
    var render = bind(div);
    render(_templateObject2, null);
    matchInnerHTML('<p></p>', div);
  });

  it('resists XSS by escaping html', function () {
    var render = bind(div);
    render(_templateObject2, '<script>alert("boom");</script>');
    matchInnerHTML('<p>&lt;script&gt;alert("boom");&lt;/script&gt;</p>', div);
  });

  it('renders raw html if explicitly told', function () {
    var render = bind(div);
    render(_templateObject2, { html: '<em>foo</em>' });
    matchInnerHTML('<p>' + '<em>foo</em>' + '</p>', div);
  });

  it('injects a node in-place', function () {
    var render = bind(div);
    var em = document.createElement('em');
    em.textContent = 'foo';
    render(_templateObject2, em);
    matchInnerHTML('<p><em>foo</em></p>', div);
  });

  it('renders element collections', function () {
    var render = bind(div);
    var span = document.createElement('span');
    span.innerHTML = '<p>Hello</p><p>friend!</p>';
    render(_templateObject3, span.childNodes);
    matchInnerHTML('<p>Hello</p><p>friend!</p>', div);
  });

  it('handles leading and trailing text', function () {
    var render = bind(div);
    render(_templateObject4, 1, 2, 3, 4);
    matchInnerHTML('1<p>2</p>3<hr>4', div);
  });

  context('performance: ', function () {
    it('re-uses the same element on re-render', function () {
      var render = bind(div);
      render(_templateObject2, 'foo');
      var p = div.querySelector('p');
      var text = p.firstChild;
      assert.isOk(p);
      assert.isOk(text);

      render(_templateObject2, 'bar');
      var newP = div.querySelector('p');
      assert.equal(p, newP);
      assert.equal(text, newP.firstChild);

      matchInnerHTML('<p>' + 'bar' + '</p>', div);
    });
  });

  context('whitespace', function () {
    it('leaves \'normal\' whitespace as is', function () {
      var render = bind(div);
      render(_templateObject5, 'foo', 'bar');
      matchInnerHTML('<p> foo bar </p>', div);
    });

    it('removes whitespace if it starts with a newline', function () {
      var render = bind(div);
      render(_templateObject6);
      matchInnerHTML('<p>Hello</p><p>Friend</p>', div);
    });

    it('wire returns a single node if possible', function () {
      var render = wire();
      var p = render(_templateObject7);
      assert.isNotArray(p);
    });

    it('wire returns an array if necessary', function () {
      var render = wire();
      var p = render(_templateObject8);
      assert.isArray(p);
    });

    it('wire does not remove \'normal\' whitespace', function () {
      var render = wire();
      var p = render(_templateObject8); //          ^ Whitespace TEXT_NODE
      //     1      2     3
      assert.equal(p.length, 3);
    });
  });

  context('edge cases', function () {
    it('can render style tags', function () {
      var render = bind(div);
      render(_templateObject9, 1);
      matchInnerHTML('<style>1</style>', div);
    });

    it('can render text around the placeholder', function () {
      var render = bind(div);
      render(_templateObject10, 1);
      matchInnerHTML('<style>Hello1Friend</style>', div);
    });

    it('can fill two parts', function () {
      var render = bind(div);
      render(_templateObject11, 'Hello', 'Friend', 'Hello', 'Friend');
      matchInnerHTML('<style>HelloXFriendXHelloXFriend</style>', div);
    });

    it('can render within textarea tags', function () {
      var render = bind(div);
      render(_templateObject12, 1);
      matchInnerHTML('<textarea>1</textarea>', div);
    });
  });

  context('arrays as values', function () {
    it('renders an array of numbers', function () {
      var render = bind(div);
      render(_templateObject2, [1, 2, 3]);
      matchInnerHTML('<p>123</p>', div);
    });

    it('renders an array of strings', function () {
      var render = bind(div);
      render(_templateObject2, ['a', 'b', 'c']);
      matchInnerHTML('<p>abc</p>', div);
    });

    it('resists XSS by escaping html', function () {
      var render = bind(div);
      render(_templateObject2, ['<em>a</em>']);
      matchInnerHTML('<p>' + '&lt;em&gt;a&lt;/em&gt;' + '</p>', div);
    });

    // IMO this is the only relevant feature here
    it('injects multiple nodes in-place', function () {
      var render = bind(div);
      var em = document.createElement('em');
      em.textContent = 'foo';
      var em2 = em.cloneNode(true);
      render(_templateObject2, [em, em2]);
      matchInnerHTML('<p><em>foo</em><em>foo</em></p>', div);
    });
  });

  context('attributes', function () {
    it('passes an attribute value', function () {
      var render = bind(div);
      render(_templateObject13, 'value');
      matchInnerHTML('<p some-attr="value"></p>', div);
    });

    it('resists XSS attacks', function () {
      var render = bind(div);
      render(_templateObject14, '"><script>alert("boo");</script><div foo="');
      matchInnerHTML('<p some-attr="&quot;><script>alert(&quot;boo&quot;);</script><div foo=&quot;"></p>', div);
    });

    it('handles `true` as setting the empty string', function () {
      var render = bind(div);
      render(_templateObject15, true);
      matchInnerHTML('<p hidden=""></p>', div);
      assert.equal(div.firstChild.hidden, true);
    });

    it('handles `false` as remove-the-attribute intent', function () {
      var render = bind(div);
      render(_templateObject15, false);
      matchInnerHTML('<p></p>', div);
      assert.equal(div.firstChild.hidden, false);
    });

    it('handles `undefined` as remove-the-attribute intent', function () {
      var render = bind(div);
      render(_templateObject15, undefined);
      matchInnerHTML('<p></p>', div);
      assert.equal(div.firstChild.hidden, false);
    });

    it('handles `null` as remove-the-attribute intent', function () {
      var render = bind(div);
      render(_templateObject15, null);
      matchInnerHTML('<p></p>', div);
      assert.equal(div.firstChild.hidden, false);
    });
  });

  context('native properties/accessors', function () {
    it('handles `null` as setting the empty string', function () {
      var render = bind(div);
      render(_templateObject16, null);
      assert.equal(div.firstChild.align, '');
      matchInnerHTML('<p align=""></p>', div);
    });

    it('handles `undefined` as setting the empty string', function () {
      var render = bind(div);
      render(_templateObject16, undefined);
      assert.equal(div.firstChild.align, '');
      matchInnerHTML('<p align=""></p>', div);
    });
  });

  context('handling events', function () {
    it('adds an event listener', function (done) {
      var render = bind(div);
      var handler = function handler(ev) {
        done();
      };
      render(_templateObject17, handler);
      var p = div.firstChild;
      var event = document.createEvent('Event');
      event.initEvent('click', false, false);
      p.dispatchEvent(event);
    });

    it('removes an event listener', function () {
      var render = bind(div);
      var handler = function handler(ev) {
        assert.fail();
      };
      render(_templateObject17, handler);
      var p = div.firstChild;
      render(_templateObject17, null);

      var event = document.createEvent('Event');
      event.initEvent('click', false, false);
      p.dispatchEvent(event);
    });
  });
});

describe('wire', function () {
  it('every call to wire() gives us new nodes', function () {
    function Title(title) {
      return wire()(_templateObject18, title);
    }

    var t1 = Title('Hello');
    var t2 = Title('World!');

    assert.notEqual(t1, t2);
  });

  it('reusing a wire updates nodes', function () {
    var render = wire();
    function Title(title) {
      return render(_templateObject18, title);
    }

    var t1 = Title('Hello');
    var t2 = Title('World!');

    assert.equal(t1, t2);
  });
});

describe('bugs', function () {
  var div = void 0;

  beforeEach(function () {
    div = document.createElement('div');
    document.body.appendChild(div);
  });

  afterEach(function () {
    document.body.removeChild(div);
  });

  it('correctly imports nodes so that custom elements v0 upgrade', function () {
    var proto = Object.create(HTMLElement.prototype);
    proto.createdCallback = function () {
      this.innerHTML = '<p>Hello</p>';
    };

    document.registerElement('x-foo', { prototype: proto });

    var render = bind(div);
    render(_templateObject19);
    assert.equal('<x-foo><p>Hello</p></x-foo>', div.innerHTML);
  });

  it('does not upgrade v1 custom elements with internal hyper attributes', function (done) {
    var XBar = function (_HTMLElement) {
      inherits(XBar, _HTMLElement);

      function XBar() {
        classCallCheck(this, XBar);
        return possibleConstructorReturn(this, (XBar.__proto__ || Object.getPrototypeOf(XBar)).apply(this, arguments));
      }

      createClass(XBar, [{
        key: 'attributeChangedCallback',
        value: function attributeChangedCallback(name, oldVal, val) {
          assert.equal(name, 'foo');
          assert.equal(val, 'bar');
          done();
        }
      }], [{
        key: 'observedAttributes',
        get: function get$$1() {
          return ['foo'];
        }
      }]);
      return XBar;
    }(HTMLElement);

    window.customElements.define('x-bar', XBar);

    var render = bind(div);
    render(_templateObject20, 'bar');
  });
});

var _templateObject$2 = taggedTemplateLiteral(['<p my-attr$=', '></p>'], ['<p my-attr$=', '></p>']);
var _templateObject2$1 = taggedTemplateLiteral(['<p on-click=', '></p>'], ['<p on-click=', '></p>']);
var _templateObject3$1 = taggedTemplateLiteral(['<p>', '</p>'], ['<p>', '</p>']);

/* global chai, Rx */

// import '../node_modules/rx-lite/rx.lite.js'

var _chai$2 = chai;
var assert$2 = _chai$2.assert;


describe('observers in attributes', function () {
  var div = void 0;

  beforeEach(function () {
    div = document.createElement('div');
    document.body.appendChild(div);
  });

  afterEach(function () {
    document.body.removeChild(div);
  });

  it('updates on pushed values', function () {
    var subject = new Rx.Subject();

    var render = bind(div);
    render(_templateObject$2, subject);
    matchInnerHTML('<p></p>', div);
    subject.next(1);
    matchInnerHTML('<p my-attr="1"></p>', div);
  });

  it('switches to next passed observable and unsubscribes', function () {
    var subject = new Rx.Subject();

    var render = bind(div);
    render(_templateObject$2, subject);
    subject.next(1);
    matchInnerHTML('<p my-attr="1"></p>', div);

    var nextSubject = new Rx.Subject();
    render(_templateObject$2, nextSubject);
    nextSubject.next('foo');
    subject.next(2);
    matchInnerHTML('<p my-attr="foo"></p>', div);

    render(_templateObject$2, null);
    assert$2.equal(subject.observers.length, 0);
  });
});

describe('event handlers as subjects', function () {
  var div = void 0;

  beforeEach(function () {
    div = document.createElement('div');
    document.body.appendChild(div);
  });

  afterEach(function () {
    document.body.removeChild(div);
  });

  it('pushes the event out to the subject', function (done) {
    var clicks = new Rx.Subject();
    clicks.subscribe(function (ev) {
      assert$2.equal(ev.target, div.firstChild);
      done();
    });

    var render = bind(div);
    render(_templateObject2$1, clicks);
    var p = div.firstChild;
    var event = document.createEvent('Event');
    event.initEvent('click', false, false);
    p.dispatchEvent(event);
  });

  it('removes the event listener when no subject listens', function (done) {
    var clicks = new Rx.Subject();

    var render = bind(div);
    render(_templateObject2$1, clicks);

    var p = div.firstChild;
    p.removeEventListener = function () {
      done();
    };

    render(_templateObject2$1, null);
  });
});

describe('observers for nodes', function () {
  var div = void 0;

  beforeEach(function () {
    div = document.createElement('div');
    document.body.appendChild(div);
  });

  afterEach(function () {
    document.body.removeChild(div);
  });

  it('pushes values into the template hole', function () {
    var subject = new Rx.Subject();

    var render = bind(div);
    render(_templateObject3$1, subject);

    matchInnerHTML('<p></p>', div);

    subject.next('Hello');
    matchInnerHTML('<p>Hello</p>', div);
  });

  it('unsubscribes a previous observable', function () {
    var render = bind(div);

    var subject = new Rx.Subject();
    render(_templateObject3$1, subject);
    var newSubject = new Rx.Subject();
    render(_templateObject3$1, newSubject);

    subject.next('Hello');
    matchInnerHTML('<p></p>', div);

    newSubject.next('World');
    matchInnerHTML('<p>World</p>', div);
  });
});

var _templateObject$3 = taggedTemplateLiteral(['<h1>', '</h1>'], ['<h1>', '</h1>']);
var _templateObject2$2 = taggedTemplateLiteral(['<line y1="1" y2="1"/>'], ['<line y1="1" y2="1"/>']);

/* global chai */

var _chai$3 = chai;
var assert$3 = _chai$3.assert;


describe('new wire', function () {
  it('every call to wire() gives us new nodes', function () {
    function Title(title) {
      return wireHtml()(_templateObject$3, title);
    }

    var t1 = Title('Hello');
    var t2 = Title('World!');

    assert$3.notEqual(t1, t2);
  });

  it('reusing a wire updates nodes', function () {
    var render = wireHtml();
    function Title(title) {
      return render(_templateObject$3, title);
    }

    var t1 = Title('Hello');
    var t2 = Title('World!');

    assert$3.equal(t1, t2);
  });

  describe('wire svg', function () {
    it('works?', function () {
      var line = wireSvg()(_templateObject2$2);

      assert$3.equal(line.tagName, 'line');
      assert$3.equal(line.namespaceURI, 'http://www.w3.org/2000/svg');
    });
  });

  describe('bind svg', function () {
    var svg$$1 = void 0;

    beforeEach(function () {
      svg$$1 = document.createElement('svg');
      document.body.appendChild(svg$$1);
    });

    afterEach(function () {
      document.body.removeChild(svg$$1);
    });

    it('works?', function () {
      var render = bind(svg$$1);
      render(_templateObject2$2);

      matchInnerHTML('<line y1="1" y2="1"></line>', svg$$1);
    });
  });
});

var _templateObject$4 = taggedTemplateLiteral(['Hello friend!'], ['Hello friend!']);
var _templateObject2$3 = taggedTemplateLiteral(['', ''], ['', '']);
var _templateObject3$2 = taggedTemplateLiteral(['<div>Hello</div><div>friend!</div>'], ['<div>Hello</div><div>friend!</div>']);
var _templateObject4$1 = taggedTemplateLiteral(['<p>Hello ', '!</p>'], ['<p>Hello ', '!</p>']);
var _templateObject5$1 = taggedTemplateLiteral(['', '', ''], ['', '', '']);
var _templateObject6$1 = taggedTemplateLiteral(['<p>Hello</p><p>', '!</p>'], ['<p>Hello</p><p>', '!</p>']);

/* global chai */

var _chai$4 = chai;
var assert$4 = _chai$4.assert;


describe('html tag', function () {
  var div = void 0;

  beforeEach(function () {
    div = document.createElement('div');
    document.body.appendChild(div);
  });

  afterEach(function () {
    document.body.removeChild(div);
  });

  it('can be materialized', function () {
    var greeting = html(_templateObject$4);
    var $text = materialize(greeting);

    assert$4.equal('Hello friend!', $text.textContent);
  });

  it('can be rendered', function () {
    var greeting = html(_templateObject$4);
    var render = bind(div);
    render(_templateObject2$3, greeting);

    matchInnerHTML('Hello friend!', div);
  });

  it('can render multiple nodes', function () {
    var greeting = html(_templateObject3$2);
    var render = bind(div);
    render(_templateObject2$3, greeting);

    matchInnerHTML('<div>Hello</div><div>friend!</div>', div);
  });

  it('can render two of the same', function () {
    var greeting = function greeting() {
      var name = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : 'friend';
      return html(_templateObject4$1, name);
    };
    var render = bind(div);
    render(_templateObject5$1, greeting(), greeting());

    matchInnerHTML('<p>Hello ' + 'friend' + '!</p><p>Hello ' + 'friend' + '!</p>', div);
    var p = div.firstElementChild;
    render(_templateObject5$1, greeting('you'), greeting());
    matchInnerHTML('<p>Hello ' + 'you' + '!</p><p>Hello ' + 'friend' + '!</p>', div);
    assert$4.equal(p, div.firstElementChild);
  });

  it('can render an array', function () {
    var greeting = function greeting() {
      var name = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : 'friend';
      return html(_templateObject4$1, name);
    };
    var friends = ['Tom', 'Bartels'];
    var render = bind(div);
    render(_templateObject2$3, friends.map(greeting));
    var p = div.firstElementChild;

    matchInnerHTML('<p>Hello Tom!</p><p>Hello Bartels!</p>', div);

    friends = ['Henry', 'Bartels'];
    render(_templateObject2$3, friends.map(greeting));

    matchInnerHTML('<p>Hello Henry!</p><p>Hello Bartels!</p>', div);
    assert$4.equal(p, div.firstElementChild);
  });

  it('can render array of multiple nodes', function () {
    var greeting = function greeting() {
      var name = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : 'friend';
      return html(_templateObject6$1, name);
    };
    var friends = ['Tom'];
    var render = bind(div);
    render(_templateObject2$3, friends.map(greeting));
    matchInnerHTML('<p>Hello</p><p>Tom!</p>', div);
  });

  it('clears the space on empty array', function () {
    var greeting = function greeting() {
      var name = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : 'friend';
      return html(_templateObject4$1, name);
    };
    var friends = ['Tom', 'Bartels'];
    var render = bind(div);
    render(_templateObject2$3, friends.map(greeting));

    friends = [];
    render(_templateObject2$3, friends.map(greeting));

    matchInnerHTML('', div);
  });

  it('render supports keys for efficient updates (1 - using `html(key)`)', function () {
    var friends = ['Tom', 'Bartels'];
    var render = bind(div);
    render(_templateObject2$3, friends.map(function (name) {
      return html(name)(_templateObject4$1, name);
    }));
    matchInnerHTML('<p>Hello Tom!</p><p>Hello Bartels!</p>', div);
    var $tom = div.firstElementChild;
    var $bartels = div.lastElementChild;

    friends = ['Bartels', 'Tom'];
    render(_templateObject2$3, friends.map(function (name) {
      return html(name)(_templateObject4$1, name);
    }));
    matchInnerHTML('<p>Hello Bartels!</p><p>Hello Tom!</p>', div);
    assert$4.equal($tom, div.lastElementChild);
    assert$4.equal($bartels, div.firstElementChild);
  });

  it('render supports keys for efficient updates (2 - using `keyed`)', function () {
    var friends = ['Tom', 'Bartels'];
    var render = bind(div);
    render(_templateObject2$3, friends.map(function (name) {
      return keyed(name, html(_templateObject4$1, name));
    }));
    matchInnerHTML('<p>Hello Tom!</p><p>Hello Bartels!</p>', div);
    var $tom = div.firstElementChild;
    var $bartels = div.lastElementChild;

    friends = ['Bartels', 'Tom'];
    render(_templateObject2$3, friends.map(function (name) {
      return keyed(name, html(_templateObject4$1, name));
    }));
    matchInnerHTML('<p>Hello Bartels!</p><p>Hello Tom!</p>', div);
    assert$4.equal($tom, div.lastElementChild);
    assert$4.equal($bartels, div.firstElementChild);
  });
});
//# sourceMappingURL=index.js.map
